package org.cebs.corep.xbrl.log;

import java.net.URL;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import de.bundesbank.bw.util.ErrorConsumer;
import de.bundesbank.util.log.ClientContext;
import de.bundesbank.util.log.ConfigurationException;
import de.bundesbank.util.log.DefaultLogger;

/**
 * EuroLogger-Klasse. Delegiert Aufrufe an DefaultLogger aus Stuttgart.
 */
public abstract class CorepLogger extends DefaultLogger
{

  // initalisiere die Konfiguration einmalig im statischen Block
  static {
    try
    {
      // hole mittels des JNDI Namens die URL der Konfigurationsdatei des DefaultLoggers
      URL url = Thread.currentThread().getContextClassLoader().getResource("de/bundesbank/util/log/LogWASConfig.xml");

      // setze die URL
      setConfigURL(url);
    }
	catch (ConfigurationException e)
    {
      System.out.println("Fehler in EuroLogger: Konfigurationsdatei nicht setzbar");
    }
  }
  /**
   * Blinde Methode, wird nie gerufen, ist notwendig, 
   * da DefaultLogger Behandlung der ConfigurationException verlangt.
   * @exception ConfigurationException 
   */
  private CorepLogger() throws ConfigurationException
  {
    // initilisiere den Default Logger 
    super();

  }
  /**
   * Zentrale Methode, die an den DefaultLogger des LogService delegiert.
   * Dieser versucht den Socket zu beschreiben. 
   * Im Falle des Scheiterns wird ein zweiter Versuch unternommen.
   * Methode ist weitgehend aus BW uebernommen.
   * @param key String
   * @param className String
   * @param methodName String
   * @param variables Object
   * @param message String
   * @param th Throwable 
   * @param cc ClientContext
  */
  public static final void log(String key, String className, String methodName, Object variables, String message, Throwable th, ClientContext cc)
  {

    // lokale Variable 
    String contextDescription = null;
    String relatedId = null;
    String error = null;

    // initialisiere Beschreibung und LogId, wenn Kontext vorliegt
    if (cc != null)
    {
      contextDescription = cc.toString();
      relatedId = cc.getLogID();
    }

    // versuche den Socket zu beschreiben
    try
    {
      //logge mit related id
      relatedId = log(key, className, methodName, variables, message, contextDescription, th, relatedId);

      // setze die LogId, wenn Kontext vorliegt
      if (cc != null)
      {
        cc.setLogID(relatedId);
      }
    }
    catch (Throwable t)
    {
      error = "Fehler in CorepLogger: Logweitergabe gescheitert";
      ErrorConsumer.handleError("Corep_XBRL", "CorepLogger", th, error, ErrorConsumer.LEVEL_ERROR);

      // wenn der erste Versuch scheiter, Logger zurcksetzen und nochmals versuchen
      getDefaultLogger().reset();

      // zweiter Versuch
      try
      {
        //logge mit related id
        relatedId = log(key, className, methodName, variables, message, contextDescription, th, relatedId);

        // setze die LogId, wenn Kontext vorliegt
        if (cc != null)
        {
          cc.setLogID(relatedId);
        }
      }
      catch (Throwable tw)
      {
        error = "Fehler in CorepLogger: Wiederholung der Logweitergabe gescheitert";
        ErrorConsumer.handleError("Corep_XBRL", "CorepLogger", th, error, ErrorConsumer.LEVEL_ERROR);
      }
    }
  }

  /**
  * Delegiert an log(String, String, String, Object, String, Throwable, ClientContext)
  * @param key String
  * @param className String
  * @param methodName String
  * @param cc ClientContext
  */
  public static final void log(String key, String className, String methodName, ClientContext cc)
  {
    log(key, className, methodName, null, null, null, cc);
  }

  /**
  	* Delegiert an log(String, String, String, Object, String, Throwable, ClientContext)
  	* @param key String
  	* @param className String
  	* @param methodName String
  	* @param cc ClientContext
  	*/
  public static final void log(String key, String className, String methodName, Object object, String message, ClientContext cc)
  {
    log(key, className, methodName, object, message, null, cc);
  }

}