package org.cebs.corep.xbrl.util;

import java.io.InputStream;
import java.net.URL;
import java.util.Enumeration;
import java.util.Properties;

import javax.naming.InitialContext;

import org.cebs.corep.xbrl.exceptions.CorepPropertyNotFoundException;

/**
 * Klasse zur Bereitstellung der System- und Applikationsparameters aus einer Properties-Datei
 * 
 * Dabei werden die Properties aus einer URL gelesen. Der JNDI-Name dieser URL kann entweder durch
 * <ol>
 * 	<li> das System-property de.bundesbank.bakism.basis.util.BakisProperties.JNDI oder
 * 	<li> eine Properties-Datei de/bundesbank/bakism/basis/util/BakisProperties.properties mit dem
 *     Property de.bundesbank.bakism.basis.util.BakisProperties.JNDI definiert werden.
 *  <li> den Default bakism/configuration wenn die oberen beiden nicht vorhanden sind
 * </ol>
 * 
 * @author: Thomas Brll
 */
public class CorepProperties
{
  // Hier werden die Properties gehalten
  private Properties props = null;

  /**
   * Liefere den JNDI-Namen der Konfiguration
   * 
   * @return String mit dem JNDI-Namen
   **/
  private String getConfigJNDI()
  {
    return "corep/configuration";
  }

  /**
   * Lese ein Property
   * 
   * @param propertyName String Name des Properties
   * @return String Die Property
   * @exception BakisPropertyNotFoundException Wird geworfen, wenn entweder die Property 
   *            nicht vorhanden oder die Property URL nicht lesbar sind
  */
  public String getProperty(String propertyName) throws CorepPropertyNotFoundException
  {

    String result = getProperties().getProperty(propertyName);
    if (result == null)
      throw new CorepPropertyNotFoundException("Property " + propertyName + " nicht gefunden!");

    return result;
  }

  /**
   * Liefert alle Properties
   * 
   * @return Returns a Properties
   */
  public Properties getProperties() throws CorepPropertyNotFoundException
  {
    // wenn Properties nicht bereits gelesen
    if (props == null)
    {
      // erzeuge Properties
      Properties p = new Properties();
      // initialisiere 
      try
      {
        // hole die URL
        InitialContext ctx = new InitialContext();
        URL url = (URL)ctx.lookup(getConfigJNDI());
        // TB: bug for Java URL on Solaris?
        if (url.getPath() == null && url.getProtocol().equals("file"))
        {
          System.out.println("Running on Solaris. not bug proven...");
          String newURL = "file://" + url.getFile();
          url = new URL(newURL);
        }
        // lade die Properties  
        InputStream in = url.openStream();
        p.load(in);
        in.close();

        // wieso werden die Properties hier nochmalig eingetragen?
        for (Enumeration keys = p.keys(); keys.hasMoreElements();)
        {
          String key = (String)keys.nextElement();
          String value = p.getProperty(key);
          p.setProperty(key, value);
        }

        // setze die Objektvariable mit der Referenz der soeben geladen Properties
        props = p;
      }
      catch (Exception e)
      {
        throw new CorepPropertyNotFoundException("Properties konnten nicht von URL gelesen werden! Ursache:" + e.getClass().getName() + " " + e.getMessage());
      }
    }
    return props;
  }

  /** Die Semi-Singleton Instanz */
  private static CorepProperties _instance = null;

  /**
   * Private Constructor
   * 
   **/
  private CorepProperties()
  {
  }

  /**
   * Hole Semi-Singleton
   * 
   **/
  public static CorepProperties getInstance()
  {
    if (_instance == null)
    {
      synchronized (CorepProperties.class)
      {
        _instance = new CorepProperties();
        // return im synchronized wegen resetInstance
        return _instance;
      }
    }
    else
      return _instance;
  }

  /**
   * Setze Semi-Singleton zurck
   * 
   **/
  public static void resetInstance()
  {
    synchronized (CorepProperties.class)
    {
      _instance = null;
    }
  }

}